work/$(DISTFILE): work cvs
	@(mkdir -p work)
	@if test ! -f work/$(DISTFILE); then \
	  cd work && wget --passive $(DISTLOCATION)/$(DISTFILE); \
	fi

## check if the root can build rpms
rpm-build:
	@if test ! -f $(BITCHESROOT)/state/rpm-build; then \
	  echo "Prepare the build root first !"; exit 1; \
	fi

## build the rpm
rpm: rpm-build pre-goals work/$(DISTFILE) rpm-prerequires rpm-buildrequires rpm-sources result-clear
	make lock
	mkdir -p $(RESULTDIR)
	@echo Starting rpm build
	@if test "x$(SPEC)" = "x"; then \
	  $(CHROOT) $(ROOT) rpm -ta /usr/src/redhat/SOURCES/$(DISTFILE) > $(RESULTDIR)/rpm.log 2>&1 || make rpm-fail; \
	else \
	  cp $(SPEC) $(ROOT); \
	  $(CHROOT) $(ROOT) chown root:root $(SPEC); \
	  $(CHROOT) $(ROOT) mv $(SPEC) /usr/src/redhat/SPECS; \
	  $(CHROOT) $(ROOT) rpm -ba /usr/src/redhat/SPECS/$(SPEC) > $(RESULTDIR)/rpm.log 2>&1 || make rpm-fail; \
	fi
	@grep "Wrote: " $(RESULTDIR)/rpm.log | cut -d\  -f2 | \
	  xargs $(CHROOT) $(ROOT) mv --target-directory=/
	mv $(ROOT)/*.rpm $(RESULTDIR)
	make unlock
	@echo "Build of $(DIST) succeeded."

## if the resultdir doesn't exist, it means we didn't build the rpm yet, no ?
$(RESULTDIR): rpm

## build rpms and install them all
rpm-install: $(RESULTDIR) rpm-requires
	make lock
	$(RPM) --root $(ROOT) -Uhv `ls $(RESULTDIR)/*.rpm | grep -v \.src\.rpm`
	make unlock

## when rpm building fails, show why and clean up
rpm-fail:
	@make unlock
	@echo
	@echo "Dump of end of rpm building log follows"
	tail -n 20 $(RESULTDIR)/rpm.log
	@echo
	@exit 1

## get the spec out of the dist
rpm-spec: work/$(DISTFILE)
## work/$(PACKAGE).spec: work/$(DISTFILE)
	@if test "x$(SPEC)" = "x"; then \
          cd work; \
          tar xzf $(DISTFILE) $(DIST)/$(PACKAGE).spec; \
	  mv $(DIST)/$(PACKAGE).spec .; \
	else \
          cp $(SPEC) work/$(PACKAGE).spec; \
	fi

## re-generate the spec file from spec.in inside the tarball by reconfiguring
rpm-spec-regen: work/$(DISTFILE) rpm-sources
	make lock
##	echo "#\!/bin/bash" > $(ROOT)/regen.sh
	echo "cd /usr/local/src && tar xzf /usr/src/redhat/SOURCES/$(DISTFILE) && \\" > $(ROOT)/regen.sh
	echo "cd $(PACKAGE)-$(VERSION) && ./configure && \\" >> $(ROOT)/regen.sh
	echo "mv $(PACKAGE).spec / && cd .." >> $(ROOT)/regen.sh
##	chmod +x $(ROOT)/regen.sh
	$(CHROOT) $(ROOT) bash regen.sh
	rm $(ROOT)/regen.sh
	@mv $(ROOT)/$(PACKAGE).spec work/$(PACKAGE).spec
	make unlock

## install all of the rpm's listed as explicit REQUIRES in the Makefile
## FIXME: we list make explicitly as to never have an empty for line which
## always fails

rpm-prerequires:
	@make lock
	@$(CHROOT) $(ROOT) apt-get update > /dev/null || echo "Could not apt-get update"
	@echo "Installing pre-requires listed in Makefile : "
	@echo "$(REQUIRES)"
	@for a in "make $(REQUIRES)"; do \
	  $(CHROOT) $(ROOT) apt-get -y install $$a; \
	done
	@make unlock

## install all of the necessary build requires
## filter out any word not starting with a letter from the buildrequires
## this line of perl works for one line but not on a pipe
## perl -ne 'print join (" ", grep {/^[a-zA-Z]\w*$/} split (/ /));'
## the bit of perl down there makes sure we get rid of version bits first,
## ie. stuff like >= alpha9.7 (cdparanoia) which would otherwise bust balls
##rpm-buildrequires: rpm-build work/$(PACKAGE).spec
rpm-buildrequires: rpm-build rpm-spec
	make lock
	@$(CHROOT) $(ROOT) apt-get update > /dev/null || echo "Could not apt-get update"
	@grep -i "^BuildRequires:" work/$(PACKAGE).spec | \
          sed 's@BuildRequires:@@' | \
          perl -pe 's/(>=|=|>|<=|<)\W*\w+//' | \
	  sed -e"s/^/ /" -e "s/[[:space:]][^a-zA-Z[:space:]][^[:space:]]*//g" | \
	  xargs $(CHROOT) $(ROOT) apt-get -y install
	make unlock

## install all of the necessary requires
## filter out any word not starting with a letter from the requires
## this line of perl works for one line but not on a pipe
## perl -ne 'print join (" ", grep {/^[a-zA-Z]\w*$/} split (/ /));'
rpm-requires: rpm-build work/$(PACKAGE).spec
	make lock
	$(CHROOT) $(ROOT) apt-get update
	@grep -i "^Requires:" work/$(PACKAGE).spec | \
          sed 's@Requires:@@' | \
	  sed -e"s/^/ /" -e "s/[[:space:]][^a-zA-Z[:space:]][^[:space:]]*//g" | \
	  xargs $(CHROOT) $(ROOT) apt-get -y install
	make unlock

## copy the dist tarball into the SOURCES dir
## copy all the files in the SOURCES/ dir to /usr/src/redhat/SOURCES
## FIXME: this is good for patches and icons and stuff, but should be
## autocollected from the tarball
rpm-sources:
	cp work/$(DISTFILE) $(ROOT)
	$(CHROOT) $(ROOT) chown root:root $(DISTFILE)
	$(CHROOT) $(ROOT) mv $(DISTFILE) /usr/src/redhat/SOURCES
	@if test -d SOURCES; then \
	  echo "+ Copying stuff from SOURCES/"; \
	  make lock; \
          cp -Lrv SOURCES $(ROOT)/tmp; \
          $(CHROOT) $(ROOT) cp -rv tmp/SOURCES usr/src/redhat; \
	  rm -rf $(ROOT)/tmp/SOURCES; \
	  make unlock; \
	fi

## pre-goals makes sure all of the listed PREGOALS get made and installed first
##
pre-goals:
	@if test "x$(PREGOALS)" != "x"; then \
          for goal in nogoal $(PREGOALS); do \
            if test -d ../$$goal; then \
	      pushd ../$$goal; make rpm-install; popd; \
	    fi; \
	  done; \
	fi

## check out stuff from cvs if CVSROOT is defined
cvs:
	@if test "x$(CVSROOT)" != "x" && test ! -f work/$(DISTFILE); then \
	  make cvs-dist; fi

cvs-dist:
	cd work && cvs -d$(CVSROOT) co $(CVSMODULE) || exit 1; \
        cd $(CVSMODULE); \
        $(AUTOGEN) || make rpm-fail; \
        $(CONFIGURE) || make rpm-fail; \
	$(MAKEDIST) || make rpm-fail; \
        mv *.tar.gz ..;

work:
	mkdir -p work

### clear the result dir for this package
result-clear:
	@if test "x$(RESULTDIR)" != "x"; then \
	  mkdir -p $(RESULTDIR); rm -rf $(RESULTDIR)/*; \
        fi

### clear the work dir
work-clear:
	rm -rf work

clear:  work-clear result-clear
